"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _S3Provider:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _S3ProviderEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_S3Provider.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AWS: _S3Provider.ValueType  # 0
    GCS: _S3Provider.ValueType  # 1

class S3Provider(_S3Provider, metaclass=_S3ProviderEnumTypeWrapper): ...

AWS: S3Provider.ValueType  # 0
GCS: S3Provider.ValueType  # 1
global___S3Provider = S3Provider

class _DspCredentialsType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DspCredentialsTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DspCredentialsType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SPLICKY: _DspCredentialsType.ValueType  # 0
    ONELOG: _DspCredentialsType.ValueType  # 1

class DspCredentialsType(_DspCredentialsType, metaclass=_DspCredentialsTypeEnumTypeWrapper): ...

SPLICKY: DspCredentialsType.ValueType  # 0
ONELOG: DspCredentialsType.ValueType  # 1
global___DspCredentialsType = DspCredentialsType

@typing.final
class S3SinkWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENDPOINT_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    OBJECTS_FIELD_NUMBER: builtins.int
    S3PROVIDER_FIELD_NUMBER: builtins.int
    USERDEFINEDCREDENTIALS_FIELD_NUMBER: builtins.int
    DQDSPCREDENTIALS_FIELD_NUMBER: builtins.int
    CONTENTTYPE_FIELD_NUMBER: builtins.int
    endpoint: builtins.str
    region: builtins.str
    """S3 region can be left empty for a GCS sink worker"""
    s3Provider: global___S3Provider.ValueType
    contentType: builtins.str
    """Optional content type for the S3 object"""
    @property
    def objects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___S3Object]: ...
    @property
    def userDefinedCredentials(self) -> global___UserDefinedCredentials: ...
    @property
    def dqDspCredentials(self) -> global___DqDspCredentials: ...
    def __init__(
        self,
        *,
        endpoint: builtins.str = ...,
        region: builtins.str = ...,
        objects: collections.abc.Iterable[global___S3Object] | None = ...,
        s3Provider: global___S3Provider.ValueType = ...,
        userDefinedCredentials: global___UserDefinedCredentials | None = ...,
        dqDspCredentials: global___DqDspCredentials | None = ...,
        contentType: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_contentType", b"_contentType", "contentType", b"contentType", "credentials", b"credentials", "dqDspCredentials", b"dqDspCredentials", "userDefinedCredentials", b"userDefinedCredentials"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_contentType", b"_contentType", "contentType", b"contentType", "credentials", b"credentials", "dqDspCredentials", b"dqDspCredentials", "endpoint", b"endpoint", "objects", b"objects", "region", b"region", "s3Provider", b"s3Provider", "userDefinedCredentials", b"userDefinedCredentials"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_contentType", b"_contentType"]) -> typing.Literal["contentType"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["credentials", b"credentials"]) -> typing.Literal["userDefinedCredentials", "dqDspCredentials"] | None: ...

global___S3SinkWorkerConfiguration = S3SinkWorkerConfiguration

@typing.final
class UserDefinedCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USERDEFINEDCREDENTIALS_FIELD_NUMBER: builtins.int
    userDefinedCredentials: builtins.str
    """This is the dependency node ID for the
    user provided credentials.
    """
    def __init__(
        self,
        *,
        userDefinedCredentials: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["userDefinedCredentials", b"userDefinedCredentials"]) -> None: ...

global___UserDefinedCredentials = UserDefinedCredentials

@typing.final
class DqDspCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DSPTYPE_FIELD_NUMBER: builtins.int
    dspType: global___DspCredentialsType.ValueType
    def __init__(
        self,
        *,
        dspType: global___DspCredentialsType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dspType", b"dspType"]) -> None: ...

global___DqDspCredentials = DqDspCredentials

@typing.final
class S3Object(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPENDENCY_FIELD_NUMBER: builtins.int
    ZIP_FIELD_NUMBER: builtins.int
    RAW_FIELD_NUMBER: builtins.int
    dependency: builtins.str
    @property
    def zip(self) -> global___ZipObject: ...
    @property
    def raw(self) -> global___RawObject: ...
    def __init__(
        self,
        *,
        dependency: builtins.str = ...,
        zip: global___ZipObject | None = ...,
        raw: global___RawObject | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["format", b"format", "raw", b"raw", "zip", b"zip"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dependency", b"dependency", "format", b"format", "raw", b"raw", "zip", b"zip"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["format", b"format"]) -> typing.Literal["zip", "raw"] | None: ...

global___S3Object = S3Object

@typing.final
class RawObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["key", b"key"]) -> None: ...

global___RawObject = RawObject

@typing.final
class ZipObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SINGLEFILE_FIELD_NUMBER: builtins.int
    FULLCONTENT_FIELD_NUMBER: builtins.int
    @property
    def singleFile(self) -> global___SingleFile: ...
    @property
    def fullContent(self) -> global___FullContent: ...
    def __init__(
        self,
        *,
        singleFile: global___SingleFile | None = ...,
        fullContent: global___FullContent | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["fullContent", b"fullContent", "kind", b"kind", "singleFile", b"singleFile"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["fullContent", b"fullContent", "kind", b"kind", "singleFile", b"singleFile"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["kind", b"kind"]) -> typing.Literal["singleFile", "fullContent"] | None: ...

global___ZipObject = ZipObject

@typing.final
class SingleFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    key: builtins.str
    path: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["key", b"key", "path", b"path"]) -> None: ...

global___SingleFile = SingleFile

@typing.final
class FullContent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___FullContent = FullContent

@typing.final
class S3Credentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCESSKEY_FIELD_NUMBER: builtins.int
    SECRETKEY_FIELD_NUMBER: builtins.int
    accessKey: builtins.str
    secretKey: builtins.str
    def __init__(
        self,
        *,
        accessKey: builtins.str = ...,
        secretKey: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["accessKey", b"accessKey", "secretKey", b"secretKey"]) -> None: ...

global___S3Credentials = S3Credentials



from __future__ import annotations
from typing_extensions import Self
from ..proto.release_policy_pb2 import (
    ReleasePolicy,
    ReleasePolicyV0,
    SecretSource,
    SecretStoreSource,
    AllowedComputeContext,
    DcrComputeContext,
)
from ..client import Client

class ReleasePolicyBuilder:
    def __init__(self, client: Client, dcr_id: str, manifest_hash: str):
        """
        Initialise an instance of the ReleasePolicyBuilder.

        **Parameters**:
        - `client`: Client object.
        - `dcr_id`: Unique identifier of the DataRoom.
        - `manifest_hash`: Unique identifier of the dataset for which the policy will be created.
        """
        self.client = client
        self.dcr_id = dcr_id
        self.manifest_hash = manifest_hash
        self.target_leaf_node_name = None

    def with_target_leaf_node_name(self, leaf_node_name: str) -> Self:
        """
        Set the target leaf node name.

        **Parameters**:
        - `leaf_node_name`: Name of the leaf node.
        """
        self.target_leaf_node_name = leaf_node_name
        return self

    def build(self) -> ReleasePolicy:
        """
        Build a release policy.

        **Returns**:
        - `policy_id`: Unique identifier of the release policy.
        """
        secret_id = self.client.get_dataset_encryption_key_secret_id(self.manifest_hash)
        if secret_id is None:
            raise Exception(
                f"Dataset with manifest hash {self.manifest_hash} has no associated secret"
            )

        return ReleasePolicy(
            v0=ReleasePolicyV0(
                secretSource=SecretSource(
                    secretStore=SecretStoreSource(secretId=secret_id),
                ),
                allowedComputeContext=AllowedComputeContext(
                    dcr=DcrComputeContext(
                        dataRoomId=self.dcr_id,
                        targetLeaf=self.target_leaf_node_name,
                    ),
                ),
            ),
        )



import io
import zipfile


def read_result_as_zipfile(result: bytes):
    """
    Read the given raw computation result as a `zipfile.ZipFile` object.
    Use the `read(name: str)` method on the returned object to read a specific
    file contained in the archive.

    Refer to the [official documentation](https://docs.python.org/3/library/zipfile.html)
    of the zipfile library for all the available methods.
    """
    return zipfile.ZipFile(io.BytesIO(result), "r")



from .. import data_lab, session
from ..attestation import EnclaveSpecifications, enclave_specifications
from ..client import Client, create_client
from ..data_lab import DataLabBuilder
from ..endorsement import Endorser
from ..session import Session
from ..storage import Key
from ..types import DataLabDatasetType
from . import attestation, data_science, types
from .builders import DataRoomBuilder, DataRoomCommitBuilder, GovernanceProtocol
from .compute import Noop, StaticContent
from .permission import Permissions

__docformat__ = "restructuredtext"

__pdoc__ = {
    "attestation": True,
    "authentication": True,
    "builders": False,
    "container": True,
    "graphql": False,
    "helpers": False,
    "node": True,
    "proto": False,
    "s3_sink": True,
    "data_source_s3": True,
    "data_science": True,
    "dataset_sink": True,
    "session": True,
    "sql": True,
    "types": True,
    "data_source_snowflake": True,
    "google_dv_360_sink": True,
    "azure_blob_storage": True,
    "salesforce": True,
    "data_lab": True,
    "permutive": True,
    "microsoft_dsp": True,
    "adform_dsp": True,
}

__all__ = [
    "create_client",
    "Client",
    "Session",
    "DataRoomBuilder",
    "DataRoomCommitBuilder",
    "DataLabBuilder",
    "DataLabDatasetType",
    "Permissions",
    "GovernanceProtocol",
    "enclave_specifications",
    "EnclaveSpecifications",
    "Key",
    "StaticContent",
    "Noop",
    "sql",
    "container",
    "s3_sink",
    "data_source_s3",
    "dataset_sink",
    "meta_sink",
    "google_dv_360_sink",
    "data_science",
    "storage",
    "attestation",
    "types",
    "authentication",
    "session",
    "node",
    "Endorser",
    "data_source_snowflake",
    "azure_blob_storage",
    "salesforce",
    "data_lab",
    "permutive",
    "microsoft_dsp",
    "adform_dsp",
]



from __future__ import annotations

from typing import TYPE_CHECKING, Dict, List, Optional, cast

from decentriq_dcr_compiler.schemas import RawLeafNode
from typing_extensions import Self

from ..session import Session
from .high_level_node import DataNode
from .node_definitions import NodeDefinition

if TYPE_CHECKING:
    from ..client import Client


class RawDataNodeDefinition(NodeDefinition):
    def __init__(
        self,
        name: str,
        is_required: bool,
        id: Optional[str] = None,
    ) -> None:
        """
        Initialise a `RawDataNodeDefinition`:

        **Parameters**:
        - `name`: Name of the `RawDataNodeDefinition`.
        - `is_required`: Defines if the `RawDataNodeDefinition` is required.
        - `id`: Optional id of the `RawDataNodeDefinition`.
        """
        super().__init__(name, id=id or name)
        self.is_required = is_required

    def _get_high_level_representation(self) -> Dict:
        """
        Retrieve the high level representation of the `RawDataNodeDefinition`.
        """
        raw_node = {
            "id": self.id,
            "name": self.name,
            "kind": {"leaf": {"isRequired": self.is_required, "kind": {"raw": {}}}},
        }
        return raw_node

    @classmethod
    def _from_high_level(
        cls,
        id: str,
        name: str,
        _node: RawLeafNode,
        is_required: bool,
    ) -> Self:
        """
        Instantiate a `RawDataNodeDefinition` from its high level representation.

        **Parameters**:
        - `name`: Name of the `RawDataNodeDefinition`.
        - `_node`: Pydantic model of the `RawDataNodeDefinition`.
        - `is_required`: Flag determining if the `RawDataNodeDefinition` must be present for dependent computations.
        """
        return cls(
            id=id,
            name=name,
            is_required=is_required,
        )

    @property
    def required_workers(self):
        return []

    def build(
        self,
        dcr_id: str,
        node_definition: NodeDefinition,
        client: Client,
        session: Session,
    ) -> RawDataNode:
        """
        Construct a RawDataNode from the Node Definition.

        **Parameters**:
        - `dcr_id`: ID of the DCR the node is a member of.
        - `node_definition`: Definition of the Raw Data Node.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `session`: The session with which to communicate with the enclave.
        """
        return RawDataNode(
            name=self.name,
            is_required=self.is_required,
            dcr_id=dcr_id,
            node_definition=cast(RawDataNodeDefinition, node_definition),
            client=client,
            session=session,
            id=self.id,
        )


class RawDataNode(DataNode):
    """
    Class representing a Raw Data node.

    Data that is provisioned to a Raw Data Node is assumed to be unstructured. This means that any of the
    SQL node types cannot read from such a Data Node. This is the preferred node type for data such as images
    or binary data. It can, of course, also be used for tabular data files such as CSV or Excel. In this case,
    however, the code reading from the Data Node will have to interpret the data correctly.
    """

    def __init__(
        self,
        id: str,
        name: str,
        is_required: bool,
        dcr_id: str,
        client: Client,
        session: Session,
        node_definition: RawDataNodeDefinition,
    ) -> None:
        """
        Initialise a `RawDataNode` instance.

        **Parameters**:
        - 'id': ID of the `RawDataNode`.
        - `name`: Name of the `RawDataNode`
        - `is_required`: Flag determining if the `RawDataNode` must be present for dependent computations.
        - `dcr_id`: ID of the DCR the node is a member of.
        - `node_definition`: Definition of the Raw Data Node.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `session`: The session with which to communicate with the enclave.
        - `node_definition`: Definition of the Raw Data Node.
        """
        super().__init__(
            name=name,
            dcr_id=dcr_id,
            client=client,
            session=session,
            is_required=is_required,
            id=id,
            definition=node_definition,
        )



"""
.. include:: ../../../../decentriq_platform_docs/container_getting_started.md
___
"""

__docformat__ = "restructuredtext"
__pdoc__ = {
    "compute": False,
    "helpers": False,
    "proto": False,
}

from .compute import StaticContainerCompute
from .helpers import read_result_as_zipfile

__all__ = [
    "StaticContainerCompute",
    "read_result_as_zipfile",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import attestation_pb2
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ComputeNodeFormat:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ComputeNodeFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ComputeNodeFormat.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    RAW: _ComputeNodeFormat.ValueType  # 0
    ZIP: _ComputeNodeFormat.ValueType  # 1

class ComputeNodeFormat(_ComputeNodeFormat, metaclass=_ComputeNodeFormatEnumTypeWrapper): ...

RAW: ComputeNodeFormat.ValueType  # 0
ZIP: ComputeNodeFormat.ValueType  # 1
global___ComputeNodeFormat = ComputeNodeFormat

@typing.final
class DataRoom(google.protobuf.message.Message):
    """/ To create a dataroom the user need to specify the `GovernanceProtocol`
    / and the list of `ConfigurationModification`.
    / The `GovernanceProtocol` defines how new modifications can be applied to an
    / existing data room. The list of `ConfigurationModification`s (abbreviated by
    / "Modification" in the diagram below) defined the structure of the data room
    / itself (the compute nodes, the permissions, etc...), this list makes up the
    / first `ConfigurationCommit` for the dataroom.
    /
    / ```diagram
    /  ┌──────────────────────┐
    /  │CreateDataRoomRequest │           ┌──────────────────┐
    /  │ ┌──────────────────┐ │           │ DataRoomContext  │
    /  │ │GovernanceProtocol│ │           │ ┌──────────────┐ │
    /  │ └──────────────────┘ │           │ │  Leaf nodes  │ │
    /  │ ┌─────────────┐      ├──────────▶│ │ manifest map │ │
    /  │ │ ┌───────────┴─┐    │           │ └──────────────┘ │
    /  │ └─┤┌────────────┴──┐ │           │ ┌──────────────┐ │
    /  │   └┤ Modification  │ │           │ │CommitsHistory│ │
    /  │    └───────────────┘ │           │ └──────────────┘ │
    /  └──────────────────────┘           └──────────────────┘
    / ```
    /
    / A dataroom has a corresponding `DataRoomContext` which includes a map of the
    / currently published datasets (and corresponding encryption key) and the
    / commits history which is a list of the `ConfigurationCommit` that make up
    / the data room
    /
    / ```diagram
    /  ┌──────────────────────────────┐
    /  │        CommitsHistory        │
    /  │┌────────────────────┐        │        ┌────┬─────────────────────────┐
    /  ││ ┌──────────────────┴─┐      │     ┌──│ Id │ConfigurationCommit #1   │
    /  ││ │  ┌─────────────────┴──┐   │     │  └────┴─────────────────────────┘
    /  ││ │  │ ┌──────────────────┴─┐ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │ConfigurationCommit │ │     ├──│ Id │ConfigurationCommit #2   │
    /  ││ │  │ │┌──────────────────┐│ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││    DataRoomId    ││ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │└──────────────────┘│ │     ├──│ Id │ConfigurationCommit #3   │
    /  ││ │  │ │┌──────────────────┐│ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││   DataRoomPin    ││ │     │  ┌────┬─────────────────────────┐
    /  ││ │  │ │└──────────────────┘│ │     ├──│ Id │ConfigurationCommit #4   │
    /  ││ │  │ │┌─────────────┐     │ │     │  └────┴─────────────────────────┘
    /  ││ │  │ ││ ┌───────────┴─┐   │ │     │
    /  │└─┤  │ │└─┤┌────────────┴──┐│ │     │
    /  │  └──┤ │  └┤ Modification  ││ │     │    ┌──────────────────────────┐
    /  │     └─┤   └───────────────┘│ │     └───▶│       DataRoomPin        │
    /  │       └────────────────────┘ │          └──────────────────────────┘
    /  └──────────────────────────────┘
    / ```
    /
    / Each `ConfigurationCommit` is identified by the data room which it refers to
    / and the pin of the data room itself. A commit can only be merged if the
    / `pin` corresponds to the current configuration history state of the data
    / room
    /
    / ```diagram
    /  ┌───────────────────────────────┐           ┌────────────────┐
    /  │MergeConfigurationCommitRequest│           │ CommitsHistory │
    /  │ ┌───────────────────────────┐ │           └────────────────┘
    /  │ │  ConfigurationCommit #5   │ │              ┌────┐
    /  │ └───────────────────────────┘ │              │ ┌──┴─┐
    /  │ ┌─────────────────────┐       ├──────────▶   └─┤ ┌──┴─┐
    /  │ │  ┌──────────────────┴──┐    │                └─┤ ┌──┴─┐
    /  │ └──┤  ┌──────────────────┴──┐ │                  └─┤ ┌──┴─┐
    /  │    └──┤   MergeSignature    │ │                    └─┤ 5  │
    /  │       └─────────────────────┘ │                      └────┘
    /  └───────────────────────────────┘
    / ```
    /
    / When a user collects the approval signatures required to merge a commit,
    / this can be added to the commits history. The list of approvers depends both
    / on the commit and the `GovernanceProtocol` that the data room is configured
    / with:
    / - StaticDataRoomPolicy: the data room is static, there isn't an approver
    / that
    /    can authorize the merge
    / - AffectedDataOwnersApprovePolicy: the approvers for a commit are the user
    / which have
    /     a CrudPermisison on the leaf node that the new nodes may use
    /
    / ```diagram
    /                                           ┌────────────────────┐
    /                                           │   UserPermission   │
    /  ┌ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐      │ ┌───────────────┐  │
    /            New Commit Node                │ │user1@email.com│──┼──┐
    /  │          ┌───┬───────┐          │      │ └───────────────┘  │  │
    /           ┌▶│0x4│Python │◀─┐              │ ┌──────────────┐   │  │
    /  │        │ └───┴───────┘  │       │      │ │ ┌────────────┴─┐ │  │
    /   ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─│─ ─ ─ ─       │ └─┤CrudPermission│ │  │
    /   ┌───────────────┐
    /  ┌ ─ ─ ─ ─│─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐      │   │   id: 0x1    │ │  └────▶│
    /  Required    │
    /     ┌───┬─┴─────┐          │              │   └──────────────┘ │        │
    /     Approvers   │
    /  │  │0x3│  Sql  │          │       │      └────────────────────┘
    /  ├───────────────┤
    /     └───┴───────┘          │              ┌────────────────────┐
    /     ┌────▶│user1@email.com│
    /  │        ▲                │       │      │   UserPermission   │  │     │ │
    /     ┌───┬─┴─────┐   ┌───┬──┴────┐         │ ┌───────────────┐  │  │
    /     │user2@email.com│
    /  │  │0x1│Leaf 1 │   │0x2│Leaf 2 │  │      │ │user2@email.com│──┼──┘
    /  └───────────────┘
    /     └───┴───────┘   └───┴───────┘         │ └───────────────┘  │
    /  │                                 │      │ ┌──────────────┐   │
    /      Old data room configuration          │ │ ┌────────────┴─┐ │
    /  └ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘      │ └─┤CrudPermission│ │
    /                                           │   │   id: 0x2    │ │
    /                                           │   └──────────────┘ │
    /                                           └────────────────────┘
    / ```
    /
    / In the example above `user3@email.com` has created a new commit which adds
    / node `0x4` to the data room. By walking the computation graph the node
    / depends on node `0x2` directly and node `0x1` indirectly. To authorize the
    / merge both owners of these leaf nodes must provide an approval signature.
    / The ownership of the nodes is determined by checking who has
    / `CrudPermission` on the leaf nodes.
    /
    / Configuration commits don't have to be merged into a data room configuration
    / history, but can also be used to execute computations on top of data the
    / user already has access to. This is determined by checking if a user has
    / ExecuteComputePermission on the node which the new nodes depend on
    /
    / ```diagram
    /  ┌ ─ ─ ─ ─ ─ ─ ─ ─ ┐
    /    New Commit Node
    /  │  ┌───┬───────┐  │
    /     │0x3│Python │       ┌────────────────────┐
    /  │  └───┴───────┘  │    │   UserPermission   │
    /   ─ ─ ─ ─ ▲ ─ ─ ─ ─     │ ┌───────────────┐  │
    /  ┌ ─ ─ ─ ─│─ ─ ─ ─ ┐    │ │user2@email.com│  │
    /     ┌───┬─┴─────┐       │ └───────────────┘  │
    /  │  │0x2│  Sql  │  │    │ ┌──────────────┐   │
    /     └───┴───────┘       │ │ ┌────────────┴─┐ │
    /  │        ▲        │    │ └─┤ExecuteCompute│ │
    /     ┌───┬─┴─────┐       │   │   id: 0x2    │ │
    /  │  │0x1│Leaf 1 │  │    │   └──────────────┘ │
    /     └───┴───────┘       └────────────────────┘
    /  │  Old data room  │
    /     configuration
    /  └ ─ ─ ─ ─ ─ ─ ─ ─ ┘
    / ```
    /
    / In the example above `user2@email.com` has created a new commit which adds
    / node `0x3` to the data room. The node has a dependency on `0x2`, for which
    / the user has `ExecuteComputePermission` which means that the user can
    / execute the node `0x3`
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    GOVERNANCEPROTOCOL_FIELD_NUMBER: builtins.int
    INITIALCONFIGURATION_FIELD_NUMBER: builtins.int
    id: builtins.str
    name: builtins.str
    description: builtins.str
    @property
    def governanceProtocol(self) -> global___GovernanceProtocol: ...
    @property
    def initialConfiguration(self) -> global___DataRoomConfiguration: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        description: builtins.str = ...,
        governanceProtocol: global___GovernanceProtocol | None = ...,
        initialConfiguration: global___DataRoomConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["governanceProtocol", b"governanceProtocol", "initialConfiguration", b"initialConfiguration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["description", b"description", "governanceProtocol", b"governanceProtocol", "id", b"id", "initialConfiguration", b"initialConfiguration", "name", b"name"]) -> None: ...

global___DataRoom = DataRoom

@typing.final
class GovernanceProtocol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATICDATAROOMPOLICY_FIELD_NUMBER: builtins.int
    AFFECTEDDATAOWNERSAPPROVEPOLICY_FIELD_NUMBER: builtins.int
    @property
    def staticDataRoomPolicy(self) -> global___StaticDataRoomPolicy: ...
    @property
    def affectedDataOwnersApprovePolicy(self) -> global___AffectedDataOwnersApprovePolicy: ...
    def __init__(
        self,
        *,
        staticDataRoomPolicy: global___StaticDataRoomPolicy | None = ...,
        affectedDataOwnersApprovePolicy: global___AffectedDataOwnersApprovePolicy | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["affectedDataOwnersApprovePolicy", b"affectedDataOwnersApprovePolicy", "policy", b"policy", "staticDataRoomPolicy", b"staticDataRoomPolicy"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["affectedDataOwnersApprovePolicy", b"affectedDataOwnersApprovePolicy", "policy", b"policy", "staticDataRoomPolicy", b"staticDataRoomPolicy"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["policy", b"policy"]) -> typing.Literal["staticDataRoomPolicy", "affectedDataOwnersApprovePolicy"] | None: ...

global___GovernanceProtocol = GovernanceProtocol

@typing.final
class StaticDataRoomPolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StaticDataRoomPolicy = StaticDataRoomPolicy

@typing.final
class AffectedDataOwnersApprovePolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___AffectedDataOwnersApprovePolicy = AffectedDataOwnersApprovePolicy

@typing.final
class DataRoomConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENTS_FIELD_NUMBER: builtins.int
    @property
    def elements(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ConfigurationElement]: ...
    def __init__(
        self,
        *,
        elements: collections.abc.Iterable[global___ConfigurationElement] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["elements", b"elements"]) -> None: ...

global___DataRoomConfiguration = DataRoomConfiguration

@typing.final
class ConfigurationElement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    COMPUTENODE_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATION_FIELD_NUMBER: builtins.int
    USERPERMISSION_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHOD_FIELD_NUMBER: builtins.int
    id: builtins.str
    @property
    def computeNode(self) -> global___ComputeNode: ...
    @property
    def attestationSpecification(self) -> attestation_pb2.AttestationSpecification: ...
    @property
    def userPermission(self) -> global___UserPermission: ...
    @property
    def authenticationMethod(self) -> global___AuthenticationMethod: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        computeNode: global___ComputeNode | None = ...,
        attestationSpecification: attestation_pb2.AttestationSpecification | None = ...,
        userPermission: global___UserPermission | None = ...,
        authenticationMethod: global___AuthenticationMethod | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["attestationSpecification", b"attestationSpecification", "authenticationMethod", b"authenticationMethod", "computeNode", b"computeNode", "element", b"element", "userPermission", b"userPermission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attestationSpecification", b"attestationSpecification", "authenticationMethod", b"authenticationMethod", "computeNode", b"computeNode", "element", b"element", "id", b"id", "userPermission", b"userPermission"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["element", b"element"]) -> typing.Literal["computeNode", "attestationSpecification", "userPermission", "authenticationMethod"] | None: ...

global___ConfigurationElement = ConfigurationElement

@typing.final
class ConfigurationModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADD_FIELD_NUMBER: builtins.int
    CHANGE_FIELD_NUMBER: builtins.int
    DELETE_FIELD_NUMBER: builtins.int
    @property
    def add(self) -> global___AddModification: ...
    @property
    def change(self) -> global___ChangeModification: ...
    @property
    def delete(self) -> global___DeleteModification: ...
    def __init__(
        self,
        *,
        add: global___AddModification | None = ...,
        change: global___ChangeModification | None = ...,
        delete: global___DeleteModification | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["add", b"add", "change", b"change", "delete", b"delete", "modification", b"modification"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["add", b"add", "change", b"change", "delete", b"delete", "modification", b"modification"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["modification", b"modification"]) -> typing.Literal["add", "change", "delete"] | None: ...

global___ConfigurationModification = ConfigurationModification

@typing.final
class AddModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENT_FIELD_NUMBER: builtins.int
    @property
    def element(self) -> global___ConfigurationElement: ...
    def __init__(
        self,
        *,
        element: global___ConfigurationElement | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["element", b"element"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["element", b"element"]) -> None: ...

global___AddModification = AddModification

@typing.final
class ChangeModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENT_FIELD_NUMBER: builtins.int
    @property
    def element(self) -> global___ConfigurationElement: ...
    def __init__(
        self,
        *,
        element: global___ConfigurationElement | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["element", b"element"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["element", b"element"]) -> None: ...

global___ChangeModification = ChangeModification

@typing.final
class DeleteModification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id"]) -> None: ...

global___DeleteModification = DeleteModification

@typing.final
class ConfigurationCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMHISTORYPIN_FIELD_NUMBER: builtins.int
    MODIFICATIONS_FIELD_NUMBER: builtins.int
    id: builtins.str
    name: builtins.str
    dataRoomId: builtins.bytes
    dataRoomHistoryPin: builtins.bytes
    @property
    def modifications(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ConfigurationModification]: ...
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        dataRoomId: builtins.bytes = ...,
        dataRoomHistoryPin: builtins.bytes = ...,
        modifications: collections.abc.Iterable[global___ConfigurationModification] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dataRoomHistoryPin", b"dataRoomHistoryPin", "dataRoomId", b"dataRoomId", "id", b"id", "modifications", b"modifications", "name", b"name"]) -> None: ...

global___ConfigurationCommit = ConfigurationCommit

@typing.final
class WindowRateLimitingConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMEWINDOWSECONDS_FIELD_NUMBER: builtins.int
    NUMMAXEXECUTIONS_FIELD_NUMBER: builtins.int
    timeWindowSeconds: builtins.int
    numMaxExecutions: builtins.int
    def __init__(
        self,
        *,
        timeWindowSeconds: builtins.int = ...,
        numMaxExecutions: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["numMaxExecutions", b"numMaxExecutions", "timeWindowSeconds", b"timeWindowSeconds"]) -> None: ...

global___WindowRateLimitingConfig = WindowRateLimitingConfig

@typing.final
class RateLimitingConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WINDOW_FIELD_NUMBER: builtins.int
    @property
    def window(self) -> global___WindowRateLimitingConfig: ...
    def __init__(
        self,
        *,
        window: global___WindowRateLimitingConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["method", b"method", "window", b"window"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["method", b"method", "window", b"window"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["method", b"method"]) -> typing.Literal["window"] | None: ...

global___RateLimitingConfig = RateLimitingConfig

@typing.final
class ComputeNode(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODENAME_FIELD_NUMBER: builtins.int
    LEAF_FIELD_NUMBER: builtins.int
    PARAMETER_FIELD_NUMBER: builtins.int
    BRANCH_FIELD_NUMBER: builtins.int
    AIRLOCK_FIELD_NUMBER: builtins.int
    RATELIMITING_FIELD_NUMBER: builtins.int
    nodeName: builtins.str
    @property
    def leaf(self) -> global___ComputeNodeLeaf: ...
    @property
    def parameter(self) -> global___ComputeNodeParameter: ...
    @property
    def branch(self) -> global___ComputeNodeBranch: ...
    @property
    def airlock(self) -> global___ComputeNodeAirlock: ...
    @property
    def rateLimiting(self) -> global___RateLimitingConfig:
        """/ Control how often this node can be interacted with
        / in the defined time interval.
        / Imporatant: this is currently checked in the metering extension
        / as implementing it in the driver would not give us additional
        / security due to replay/reset attacks.
        / In this implementation, it only checks direct interactions with a
        / node, it won't check indirect executions (if the node was a dependency
        / of another node). This issue is even bigger in DCRs with interactivity
        / where someone could simply add a new node on top that doesn't have
        / the rate limit applied.
        """

    def __init__(
        self,
        *,
        nodeName: builtins.str = ...,
        leaf: global___ComputeNodeLeaf | None = ...,
        parameter: global___ComputeNodeParameter | None = ...,
        branch: global___ComputeNodeBranch | None = ...,
        airlock: global___ComputeNodeAirlock | None = ...,
        rateLimiting: global___RateLimitingConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_rateLimiting", b"_rateLimiting", "airlock", b"airlock", "branch", b"branch", "leaf", b"leaf", "node", b"node", "parameter", b"parameter", "rateLimiting", b"rateLimiting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_rateLimiting", b"_rateLimiting", "airlock", b"airlock", "branch", b"branch", "leaf", b"leaf", "node", b"node", "nodeName", b"nodeName", "parameter", b"parameter", "rateLimiting", b"rateLimiting"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_rateLimiting", b"_rateLimiting"]) -> typing.Literal["rateLimiting"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["node", b"node"]) -> typing.Literal["leaf", "parameter", "branch", "airlock"] | None: ...

global___ComputeNode = ComputeNode

@typing.final
class ComputeNodeLeaf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ISREQUIRED_FIELD_NUMBER: builtins.int
    isRequired: builtins.bool
    def __init__(
        self,
        *,
        isRequired: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["isRequired", b"isRequired"]) -> None: ...

global___ComputeNodeLeaf = ComputeNodeLeaf

@typing.final
class ComputeNodeParameter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ISREQUIRED_FIELD_NUMBER: builtins.int
    isRequired: builtins.bool
    def __init__(
        self,
        *,
        isRequired: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["isRequired", b"isRequired"]) -> None: ...

global___ComputeNodeParameter = ComputeNodeParameter

@typing.final
class ComputeNodeAirlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QUOTABYTES_FIELD_NUMBER: builtins.int
    AIRLOCKEDDEPENDENCY_FIELD_NUMBER: builtins.int
    quotaBytes: builtins.int
    airlockedDependency: builtins.str
    def __init__(
        self,
        *,
        quotaBytes: builtins.int = ...,
        airlockedDependency: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["airlockedDependency", b"airlockedDependency", "quotaBytes", b"quotaBytes"]) -> None: ...

global___ComputeNodeAirlock = ComputeNodeAirlock

@typing.final
class ComputeNodeProtocol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    version: builtins.int
    def __init__(
        self,
        *,
        version: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["version", b"version"]) -> None: ...

global___ComputeNodeProtocol = ComputeNodeProtocol

@typing.final
class ComputeNodeBranch(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONFIG_FIELD_NUMBER: builtins.int
    DEPENDENCIES_FIELD_NUMBER: builtins.int
    OUTPUTFORMAT_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONID_FIELD_NUMBER: builtins.int
    config: builtins.bytes
    outputFormat: global___ComputeNodeFormat.ValueType
    attestationSpecificationId: builtins.str
    @property
    def dependencies(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def protocol(self) -> global___ComputeNodeProtocol: ...
    def __init__(
        self,
        *,
        config: builtins.bytes = ...,
        dependencies: collections.abc.Iterable[builtins.str] | None = ...,
        outputFormat: global___ComputeNodeFormat.ValueType = ...,
        protocol: global___ComputeNodeProtocol | None = ...,
        attestationSpecificationId: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["protocol", b"protocol"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["attestationSpecificationId", b"attestationSpecificationId", "config", b"config", "dependencies", b"dependencies", "outputFormat", b"outputFormat", "protocol", b"protocol"]) -> None: ...

global___ComputeNodeBranch = ComputeNodeBranch

@typing.final
class UserPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EMAIL_FIELD_NUMBER: builtins.int
    PERMISSIONS_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODID_FIELD_NUMBER: builtins.int
    email: builtins.str
    authenticationMethodId: builtins.str
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Permission]: ...
    def __init__(
        self,
        *,
        email: builtins.str = ...,
        permissions: collections.abc.Iterable[global___Permission] | None = ...,
        authenticationMethodId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["authenticationMethodId", b"authenticationMethodId", "email", b"email", "permissions", b"permissions"]) -> None: ...

global___UserPermission = UserPermission

@typing.final
class AuthenticationMethod(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PERSONALPKI_FIELD_NUMBER: builtins.int
    DQPKI_FIELD_NUMBER: builtins.int
    DCRSECRET_FIELD_NUMBER: builtins.int
    @property
    def personalPki(self) -> global___PkiPolicy: ...
    @property
    def dqPki(self) -> global___PkiPolicy: ...
    @property
    def dcrSecret(self) -> global___DcrSecretPolicy:
        """the policies below could be implemented later on
        EmailVerificationPolicy emailVerificationPolicy = 3;
        OpenIdConnectPolicy openIdConnectPolicy = 4;
        DcrSecretPolicy
        """

    def __init__(
        self,
        *,
        personalPki: global___PkiPolicy | None = ...,
        dqPki: global___PkiPolicy | None = ...,
        dcrSecret: global___DcrSecretPolicy | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcrSecret", b"dcrSecret", "dqPki", b"dqPki", "personalPki", b"personalPki"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcrSecret", b"dcrSecret", "dqPki", b"dqPki", "personalPki", b"personalPki"]) -> None: ...

global___AuthenticationMethod = AuthenticationMethod

@typing.final
class PkiPolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROOTCERTIFICATEPEM_FIELD_NUMBER: builtins.int
    rootCertificatePem: builtins.bytes
    def __init__(
        self,
        *,
        rootCertificatePem: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["rootCertificatePem", b"rootCertificatePem"]) -> None: ...

global___PkiPolicy = PkiPolicy

@typing.final
class DcrSecretPolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCRSECRETID_FIELD_NUMBER: builtins.int
    dcrSecretId: builtins.bytes
    def __init__(
        self,
        *,
        dcrSecretId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dcrSecretId", b"dcrSecretId"]) -> None: ...

global___DcrSecretPolicy = DcrSecretPolicy

@typing.final
class Permission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTECOMPUTEPERMISSION_FIELD_NUMBER: builtins.int
    LEAFCRUDPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSPERMISSION_FIELD_NUMBER: builtins.int
    DRYRUNPERMISSION_FIELD_NUMBER: builtins.int
    GENERATEMERGESIGNATUREPERMISSION_FIELD_NUMBER: builtins.int
    EXECUTEDEVELOPMENTCOMPUTEPERMISSION_FIELD_NUMBER: builtins.int
    MERGECONFIGURATIONCOMMITPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVECOMPUTERESULTPERMISSION_FIELD_NUMBER: builtins.int
    CASAUXILIARYSTATEPERMISSION_FIELD_NUMBER: builtins.int
    READAUXILIARYSTATEPERMISSION_FIELD_NUMBER: builtins.int
    @property
    def executeComputePermission(self) -> global___ExecuteComputePermission: ...
    @property
    def leafCrudPermission(self) -> global___LeafCrudPermission: ...
    @property
    def retrieveDataRoomPermission(self) -> global___RetrieveDataRoomPermission: ...
    @property
    def retrieveAuditLogPermission(self) -> global___RetrieveAuditLogPermission: ...
    @property
    def retrieveDataRoomStatusPermission(self) -> global___RetrieveDataRoomStatusPermission: ...
    @property
    def updateDataRoomStatusPermission(self) -> global___UpdateDataRoomStatusPermission: ...
    @property
    def retrievePublishedDatasetsPermission(self) -> global___RetrievePublishedDatasetsPermission: ...
    @property
    def dryRunPermission(self) -> global___DryRunPermission: ...
    @property
    def generateMergeSignaturePermission(self) -> global___GenerateMergeSignaturePermission: ...
    @property
    def executeDevelopmentComputePermission(self) -> global___ExecuteDevelopmentComputePermission: ...
    @property
    def mergeConfigurationCommitPermission(self) -> global___MergeConfigurationCommitPermission: ...
    @property
    def retrieveComputeResultPermission(self) -> global___RetrieveComputeResultPermission: ...
    @property
    def casAuxiliaryStatePermission(self) -> global___CasAuxiliaryStatePermission: ...
    @property
    def readAuxiliaryStatePermission(self) -> global___ReadAuxiliaryStatePermission: ...
    def __init__(
        self,
        *,
        executeComputePermission: global___ExecuteComputePermission | None = ...,
        leafCrudPermission: global___LeafCrudPermission | None = ...,
        retrieveDataRoomPermission: global___RetrieveDataRoomPermission | None = ...,
        retrieveAuditLogPermission: global___RetrieveAuditLogPermission | None = ...,
        retrieveDataRoomStatusPermission: global___RetrieveDataRoomStatusPermission | None = ...,
        updateDataRoomStatusPermission: global___UpdateDataRoomStatusPermission | None = ...,
        retrievePublishedDatasetsPermission: global___RetrievePublishedDatasetsPermission | None = ...,
        dryRunPermission: global___DryRunPermission | None = ...,
        generateMergeSignaturePermission: global___GenerateMergeSignaturePermission | None = ...,
        executeDevelopmentComputePermission: global___ExecuteDevelopmentComputePermission | None = ...,
        mergeConfigurationCommitPermission: global___MergeConfigurationCommitPermission | None = ...,
        retrieveComputeResultPermission: global___RetrieveComputeResultPermission | None = ...,
        casAuxiliaryStatePermission: global___CasAuxiliaryStatePermission | None = ...,
        readAuxiliaryStatePermission: global___ReadAuxiliaryStatePermission | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["casAuxiliaryStatePermission", b"casAuxiliaryStatePermission", "dryRunPermission", b"dryRunPermission", "executeComputePermission", b"executeComputePermission", "executeDevelopmentComputePermission", b"executeDevelopmentComputePermission", "generateMergeSignaturePermission", b"generateMergeSignaturePermission", "leafCrudPermission", b"leafCrudPermission", "mergeConfigurationCommitPermission", b"mergeConfigurationCommitPermission", "permission", b"permission", "readAuxiliaryStatePermission", b"readAuxiliaryStatePermission", "retrieveAuditLogPermission", b"retrieveAuditLogPermission", "retrieveComputeResultPermission", b"retrieveComputeResultPermission", "retrieveDataRoomPermission", b"retrieveDataRoomPermission", "retrieveDataRoomStatusPermission", b"retrieveDataRoomStatusPermission", "retrievePublishedDatasetsPermission", b"retrievePublishedDatasetsPermission", "updateDataRoomStatusPermission", b"updateDataRoomStatusPermission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["casAuxiliaryStatePermission", b"casAuxiliaryStatePermission", "dryRunPermission", b"dryRunPermission", "executeComputePermission", b"executeComputePermission", "executeDevelopmentComputePermission", b"executeDevelopmentComputePermission", "generateMergeSignaturePermission", b"generateMergeSignaturePermission", "leafCrudPermission", b"leafCrudPermission", "mergeConfigurationCommitPermission", b"mergeConfigurationCommitPermission", "permission", b"permission", "readAuxiliaryStatePermission", b"readAuxiliaryStatePermission", "retrieveAuditLogPermission", b"retrieveAuditLogPermission", "retrieveComputeResultPermission", b"retrieveComputeResultPermission", "retrieveDataRoomPermission", b"retrieveDataRoomPermission", "retrieveDataRoomStatusPermission", b"retrieveDataRoomStatusPermission", "retrievePublishedDatasetsPermission", b"retrievePublishedDatasetsPermission", "updateDataRoomStatusPermission", b"updateDataRoomStatusPermission"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["permission", b"permission"]) -> typing.Literal["executeComputePermission", "leafCrudPermission", "retrieveDataRoomPermission", "retrieveAuditLogPermission", "retrieveDataRoomStatusPermission", "updateDataRoomStatusPermission", "retrievePublishedDatasetsPermission", "dryRunPermission", "generateMergeSignaturePermission", "executeDevelopmentComputePermission", "mergeConfigurationCommitPermission", "retrieveComputeResultPermission", "casAuxiliaryStatePermission", "readAuxiliaryStatePermission"] | None: ...

global___Permission = Permission

@typing.final
class ExecuteComputePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMPUTENODEID_FIELD_NUMBER: builtins.int
    computeNodeId: builtins.str
    def __init__(
        self,
        *,
        computeNodeId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["computeNodeId", b"computeNodeId"]) -> None: ...

global___ExecuteComputePermission = ExecuteComputePermission

@typing.final
class LeafCrudPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEAFNODEID_FIELD_NUMBER: builtins.int
    leafNodeId: builtins.str
    def __init__(
        self,
        *,
        leafNodeId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["leafNodeId", b"leafNodeId"]) -> None: ...

global___LeafCrudPermission = LeafCrudPermission

@typing.final
class RetrieveDataRoomPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RetrieveDataRoomPermission = RetrieveDataRoomPermission

@typing.final
class RetrieveAuditLogPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RetrieveAuditLogPermission = RetrieveAuditLogPermission

@typing.final
class RetrieveDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RetrieveDataRoomStatusPermission = RetrieveDataRoomStatusPermission

@typing.final
class UpdateDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateDataRoomStatusPermission = UpdateDataRoomStatusPermission

@typing.final
class RetrievePublishedDatasetsPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RetrievePublishedDatasetsPermission = RetrievePublishedDatasetsPermission

@typing.final
class DryRunPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DryRunPermission = DryRunPermission

@typing.final
class GenerateMergeSignaturePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GenerateMergeSignaturePermission = GenerateMergeSignaturePermission

@typing.final
class ExecuteDevelopmentComputePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ExecuteDevelopmentComputePermission = ExecuteDevelopmentComputePermission

@typing.final
class MergeConfigurationCommitPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___MergeConfigurationCommitPermission = MergeConfigurationCommitPermission

@typing.final
class RetrieveComputeResultPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMPUTENODEID_FIELD_NUMBER: builtins.int
    computeNodeId: builtins.str
    def __init__(
        self,
        *,
        computeNodeId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["computeNodeId", b"computeNodeId"]) -> None: ...

global___RetrieveComputeResultPermission = RetrieveComputeResultPermission

@typing.final
class CasAuxiliaryStatePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CasAuxiliaryStatePermission = CasAuxiliaryStatePermission

@typing.final
class ReadAuxiliaryStatePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ReadAuxiliaryStatePermission = ReadAuxiliaryStatePermission



from typing import Any, Dict, List
from typing_extensions import Self


class LalAudienceStatistics:
    """
    Class representing the statistics of a given lookalike audience.
    """

    def __init__(
        self,
        auc: float,
        fpr: List[float],
        tpr: List[float],
    ) -> None:
        """
        Initialise an instance of the `LalAudienceStatistics`

        **Parameters**:
        - `auc`: The area under the ROC curve.
        - `fpr`: The list of false postive rates (in order of decreasing discrimination threshold).
        - `tpr`: The list of true positive rates (in order of decreasing discrimination threshold).
        """
        self.auc = auc
        self.fpr = fpr
        self.tpr = tpr

    def as_dict(self) -> Dict[str, Any]:
        return {
            "quality": {
                "roc_curve": {"auc": self.auc, "fpr": self.fpr, "tpr": self.tpr}
            }
        }

    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> Self:
        roc_curve = d["quality"]["roc_curve"]
        return cls(
            auc=roc_curve["auc"],
            fpr=roc_curve["fpr"],
            tpr=roc_curve["tpr"],
        )



from typing import Any, Dict, List
from typing_extensions import Self
import json


class LalAudienceStatistics:
    """
    Class representing the statistics of a given lookalike audience.
    """

    def __init__(
        self,
        auc: float,
        fpr: List[float],
        tpr: List[float],
    ) -> None:
        """
        Initialize an instance of LalAudienceStatistics.

        **Parameters**:
        - `auc`: The area under the ROC curve
        - `fpr`: The list of false positive rates (in order of decreasing discrimination threshold)
        - `tpr`: The list of true positive rates (in order of decreasing discrimination threshold)
        """
        self.auc = auc
        self.fpr = fpr
        self.tpr = tpr

    def model_dump(self) -> Dict[str, Any]:
        """
        Convert the instance to a dictionary following pydantic model_dump convention.

        **Returns**:
        - Dictionary representation of the statistics
        """
        return {
            "quality": {
                "roc_curve": {"auc": self.auc, "fpr": self.fpr, "tpr": self.tpr}
            }
        }

    def model_dump_json(self, indent: int = None) -> str:
        """
        Convert the instance to a JSON string following pydantic model_dump_json convention.

        **Parameters**:
        - `indent`: Number of spaces for indentation in the JSON output

        **Returns**:
        - JSON string representation of the statistics
        """
        return json.dumps(self.model_dump(), indent=indent)

    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> Self:
        """
        Create an instance from a dictionary.

        **Parameters**:
        - `d`: Dictionary containing the statistics data

        **Returns**:
        - New LalAudienceStatistics instance
        """
        roc_curve = d["quality"]["roc_curve"]
        return cls(
            auc=roc_curve["auc"],
            fpr=roc_curve["fpr"],
            tpr=roc_curve["tpr"],
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _S3Provider:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _S3ProviderEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_S3Provider.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AWS: _S3Provider.ValueType  # 0
    GCS: _S3Provider.ValueType  # 1

class S3Provider(_S3Provider, metaclass=_S3ProviderEnumTypeWrapper): ...

AWS: S3Provider.ValueType  # 0
GCS: S3Provider.ValueType  # 1
global___S3Provider = S3Provider

@typing.final
class S3Source(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUCKET_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    OBJECTKEY_FIELD_NUMBER: builtins.int
    bucket: builtins.str
    region: builtins.str
    """region is unused (thus, can be left empty) for GCS."""
    objectKey: builtins.str
    def __init__(
        self,
        *,
        bucket: builtins.str = ...,
        region: builtins.str = ...,
        objectKey: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["bucket", b"bucket", "objectKey", b"objectKey", "region", b"region"]) -> None: ...

global___S3Source = S3Source

@typing.final
class DataSourceS3WorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    S3PROVIDER_FIELD_NUMBER: builtins.int
    credentialsDependency: builtins.str
    s3Provider: global___S3Provider.ValueType
    @property
    def source(self) -> global___S3Source: ...
    def __init__(
        self,
        *,
        source: global___S3Source | None = ...,
        credentialsDependency: builtins.str = ...,
        s3Provider: global___S3Provider.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["source", b"source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["credentialsDependency", b"credentialsDependency", "s3Provider", b"s3Provider", "source", b"source"]) -> None: ...

global___DataSourceS3WorkerConfiguration = DataSourceS3WorkerConfiguration



from __future__ import annotations

from google.protobuf.json_format import MessageToDict

from .proto import (
    AzureBlobStorageWorkerConfiguration,
    ContainerWorkerConfiguration,
    DatasetSinkWorkerConfiguration,
    DataSourceS3WorkerConfiguration,
    DataSourceSnowflakeWorkerConfiguration,
    DriverTaskConfig,
    GoogleAdManagerWorkerConfiguration,
    GoogleDv360SinkWorkerConfiguration,
    MetaSinkWorkerConfiguration,
    PermutiveWorkerConfiguration,
    PostWorkerConfiguration,
    S3SinkWorkerConfiguration,
    SalesforceWorkerConfiguration,
    SqlWorkerConfiguration,
    MicrosoftDspWorkerConfiguration,
    AdformDspWorkerConfiguration,
    AmazonAmcWorkerConfiguration,
    parse_length_delimited,
)


class GcgDriverDecoder:
    def decode(self, config: bytes):
        config_decoded = DriverTaskConfig()
        parse_length_delimited(config, config_decoded)
        config_decoded_json = MessageToDict(config_decoded)
        if config_decoded.HasField("staticContent"):
            content = config_decoded.staticContent.content.decode("utf-8")
            config_decoded_json["staticContent"]["content"] = content
        return config_decoded_json


class PostWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = PostWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class AzureBlobStorageWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = AzureBlobStorageWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class ContainerWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = ContainerWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class DataSourceS3WorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DataSourceS3WorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class DataSourceSnowflakeWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DataSourceSnowflakeWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class DatasetSinkWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DatasetSinkWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class GoogleAdManagerWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = GoogleAdManagerWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class GoogleDv360SinkWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = GoogleDv360SinkWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class MetaSinkWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = MetaSinkWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class PermutiveWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = PermutiveWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class S3SinkWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = S3SinkWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class DataSourceSalesforceWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = SalesforceWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class SqlWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = SqlWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class MicrosoftDspWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = MicrosoftDspWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class AdformDspWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = AdformDspWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)


class AmazonAmcWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = AmazonAmcWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)



from __future__ import annotations

import io
from typing import Optional, TYPE_CHECKING

from abc import ABC, abstractmethod
import zipfile
from ..session import Session
from ..types import JobId
from decentriq_dcr_compiler.schemas import (
    AbMediaRequest,
    AbMediaResponse,
    RequestedAudiencePayload,
)
from .request import Request

if TYPE_CHECKING:
    from ..client import Client


class Computation(ABC):
    """
    Abstract class representing a computation.
    """

    def __init__(self, dcr_id: str, client: Client, session: Session) -> None:
        super().__init__()
        self.dcr_id = dcr_id
        self.client = client
        self.session = session
        self.job_id = None

    @abstractmethod
    def node_id(self) -> str:
        pass

    def send_request(self, request: AbMediaRequest) -> AbMediaResponse:
        return Request.send(request, self.session)

    def run(
        self,
        request_type: str,
        generate_audience: Optional[RequestedAudiencePayload] = None,
        lal_audience: Optional[RequestedAudiencePayload] = None,
    ) -> None:
        """
        Run the computation
        """
        request_payload = {
            "dataRoomIdHex": self.dcr_id,
            "scopeIdHex": self.client._ensure_dcr_data_scope(self.dcr_id),
        }
        if generate_audience:
            request_payload["generateAudience"] = generate_audience.model_dump(
                mode="json"
            )
        if lal_audience:
            request_payload["lalAudience"] = lal_audience.model_dump(mode="json")

        request = AbMediaRequest.model_validate({request_type: request_payload})
        response = self.send_request(request).model_dump()
        self.job_id = JobId(
            job_id=response[request_type]["jobIdHex"],
            compute_node_id=self.node_id(),
        )

    def get_results_str_from_zip(
        self, file_name: str, interval: int = 5, timeout: Optional[int] = None
    ) -> str:
        results = self.session.get_computation_result(
            self.job_id,
            interval=interval,
            timeout=timeout,
        )
        zip = zipfile.ZipFile(io.BytesIO(results), "r")
        if file_name in zip.namelist():
            return zip.read(file_name).decode()
        else:
            raise Exception(f"Failed to read {file_name} from zip.")

    def get_results(self, interval: int = 5, timeout: Optional[int] = None) -> bytes:
        pass

    def run_and_get_results(
        self, interval: int = 5, timeout: Optional[int] = None
    ) -> bytes:
        self.run()
        return self.get_results(interval=interval, timeout=timeout)

    def wait_until_computation_has_finished(self) -> None:
        self.run()
        self.session.wait_until_computation_has_finished(self.job_id)



from __future__ import annotations
import base64
import json

from typing_extensions import Self
from typing import Dict, List, Optional, TYPE_CHECKING, Tuple
from decentriq_dcr_compiler import compiler, ab_media as ab_media_compiler
from decentriq_dcr_compiler.schemas import CreateAbMediaDcr
from ..attestation import enclave_specifications
from ..types import MATCHING_ID_INTERNAL_LOOKUP, MatchingId, EnclaveSpecification
from ..proto import serialize_length_delimited
from .ab_media import AbMediaDcrDefinition
from .version import AUDIENCE_BUILDER_SUPPORTED_VERSION
from .helper import generate_id

if TYPE_CHECKING:
    from ..client import Client


class AbMediaDcrBuilder:
    """
    A helper class to build an Audience Builder Media DCR.

    Required arguments for building a DCR are passed via the constructor of this
    class. Optional arguments, instead, can be passed through the various
    builder methods.
    """

    def __init__(
        self,
        *,
        client: Client,
        name: str,
        publisher_emails: list[str],
        advertiser_emails: list[str],
        matching_id_format: MatchingId,
        enclave_specs: Optional[dict[str, EnclaveSpecification]] = None,
    ) -> None:
        """
        Initialise an Audience Builder Media DCR builder.

        **Parameters**:
        - `client`: A `Client` object that can be used to retrieve information about the platform.
        - `name`: The name of the DCR.
        - `publisher_emails`: A list with all publisher participants.
        - `advertiser_emails`: A list with all advertiser participants.
        - `matching_id_format`: The format of the matching ids, i.e. the ids that are used
            to match users between the publisher's and advertiser's datasets.
        - `enclave_specs`: Determines the types of enclaves that will supported by this Data Clean Room.
            If not specified, the latest enclave specifications known to this
            SDK version will be used.
        """
        self.client = client
        self.enclave_specs = (
            enclave_specs if enclave_specs else enclave_specifications.latest()
        )
        self.name = name
        self.main_publisher_email = publisher_emails[0]
        self.additional_publisher_emails = publisher_emails[1:]
        self.main_advertiser_email = advertiser_emails[0]
        self.additional_advertiser_emails = advertiser_emails[1:]
        self.agency_emails = []
        self.observer_emails = []
        self.data_partner_emails = []
        self.enable_data_partner = False
        self.matching_id = matching_id_format
        self.enable_insights = False
        self.enable_lookalike = False
        self.enable_remarketing = False
        self.enable_rule_based = False
        self.hide_absolute_values_from_insights = False
        self.enable_advertiser_audience_download = False

    def with_agency_emails(self, emails: List[str]) -> Self:
        """
        Set the agency email addresses.

        **Parameters**:
        - `emails`: List of agency email addresses.
        """
        self.agency_emails = emails
        return self

    def with_observer_emails(self, emails: List[str]) -> Self:
        """
        Set the observer email addresses.

        **Parameters**:
        - `emails`: List of observer email addresses.
        """
        self.observer_emails = emails
        return self

    def with_data_partner_emails(self, emails: List[str]) -> Self:
        """
        Set the data partner email addresses.

        **Parameters**:
        - `emails`: List of data partner email addresses.
        """
        self.data_partner_emails = emails
        self.enable_data_partner = True
        return self

    def with_insights(self) -> Self:
        """
        Enable the "insights" feature set.
        """
        self.enable_insights = True
        return self

    def with_lookalike(self) -> Self:
        """
        Enable the "lookalike" feature set.
        """
        self.enable_lookalike = True
        return self

    def with_remarketing(self) -> Self:
        """
        Enable the "remarketing" feature set.
        """
        self.enable_remarketing = True
        return self

    def with_rule_based(self) -> Self:
        """
        Enable the "rule-based" feature set.
        """
        self.enable_rule_based = True
        self.enable_remarketing = True
        return self

    def with_advertiser_audience_download(self) -> Self:
        """
        Allow the advertiser to download the user ids for an audience.
        """
        self.enable_advertiser_audience_download = True
        return self

    def with_hide_absolute_values_from_insights(self) -> Self:
        """
        Ensure absolute values are not included in the output of the insights computation.
        """
        self.hide_absolute_values_from_insights = True
        return self

    def build(self) -> AbMediaDcrDefinition:
        """
        Build the Data Clean Room.

        In order to use the DCR, the output of this method should be passed to
        `client.publish_media_dcr`.
        """
        self._check_advertiser_audience_download_allowed()

        id = f"Audience Builder DCR {generate_id()}"
        root_cert_pem = self.client.decentriq_ca_root_certificate.decode()
        (
            matching_id_format,
            matching_id_hashing_algorithm,
        ) = MATCHING_ID_INTERNAL_LOOKUP[self.matching_id]
        (driver_spec, python_spec) = self._get_ab_dcr_enclave_specs()

        # Dedupe the advertiser and publisher emails.
        # This ensures that the main email address only appears once even if it
        # exists in the addtional email addresses.
        advertiser_emails = list(
            set([self.main_advertiser_email] + self.additional_advertiser_emails)
        )
        publisher_emails = list(
            set([self.main_publisher_email] + self.additional_publisher_emails)
        )
        high_level = {
            AUDIENCE_BUILDER_SUPPORTED_VERSION: {
                "advertiserEmails": advertiser_emails,
                "agencyEmails": self.agency_emails,
                "authenticationRootCertificatePem": root_cert_pem,
                "dataPartnerEmails": self.data_partner_emails,
                "driverEnclaveSpecification": {
                    "attestationProtoBase64": driver_spec.attestationProtoBase64,
                    "id": driver_spec.id,
                    "workerProtocol": driver_spec.workerProtocol,
                },
                "enableAdvertiserAudienceDownload": self.enable_advertiser_audience_download,
                "enableDataPartner": self.enable_data_partner,
                "enableDebugMode": False,
                "enableHideAbsoluteValuesFromInsights": self.hide_absolute_values_from_insights,
                "enableInsights": self.enable_insights,
                "enableLookalikeAudiences": self.enable_lookalike,
                "enableRateLimitingOnPublishDataset": True,
                "enableRemarketing": self.enable_remarketing,
                "enableRuleBasedAudiences": self.enable_rule_based,
                "hashMatchingIdWith": (
                    None
                    if matching_id_hashing_algorithm is None
                    else matching_id_hashing_algorithm.value
                ),
                "id": id,
                "mainAdvertiserEmail": self.main_advertiser_email,
                "mainPublisherEmail": self.main_publisher_email,
                "matchingIdFormat": matching_id_format.value,
                "modelEvaluation": {
                    "postScopeMerge": ["ROC_CURVE"],
                    "preScopeMerge": [],
                },
                "name": self.name,
                "observerEmails": self.observer_emails,
                "publisherEmails": publisher_emails,
                "pythonEnclaveSpecification": {
                    "attestationProtoBase64": python_spec.attestationProtoBase64,
                    "id": python_spec.id,
                    "workerProtocol": python_spec.workerProtocol,
                },
            }
        }
        audience_builder_dcr = ab_media_compiler.create_ab_media_dcr(
            CreateAbMediaDcr.model_validate(high_level)
        )
        dcr_definition = AbMediaDcrDefinition(
            name=self.name,
            high_level=json.loads(audience_builder_dcr.model_dump_json(by_alias=True)),
            enclave_specs=self.enclave_specs,
        )
        return dcr_definition

    def _check_advertiser_audience_download_allowed(self):
        if not self.enable_advertiser_audience_download:
            return

        # Ensure the correct feature sets are enabled before allowing advertiser audience downloads.
        if self.enable_rule_based:
            raise Exception(
                "Advertiser audience downloads are not permitted when rule-based audiences are enabled"
            )
        elif not self.enable_lookalike and not self.enable_remarketing:
            raise Exception(
                "Advertiser audience downloads require the lookalike or remarketing feature to be enabled"
            )

    def _get_ab_dcr_enclave_specs(
        self,
    ) -> Tuple[compiler.EnclaveSpecification, compiler.EnclaveSpecification]:
        driver_spec = None
        python_spec = None
        for spec_id, spec in self.enclave_specs.items():
            spec_payload = {
                "attestationProtoBase64": base64.b64encode(
                    serialize_length_delimited(spec["proto"])
                ).decode(),
                "id": spec_id,
                "workerProtocol": spec["workerProtocols"][0],
            }
            if "decentriq.driver" in spec_id:
                driver_spec = compiler.EnclaveSpecification.model_validate(spec_payload)
            elif "decentriq.python-ml-worker" in spec_id:
                python_spec = compiler.EnclaveSpecification.model_validate(spec_payload)
        if driver_spec is None:
            raise Exception("No driver enclave spec found for the datalab")
        if python_spec is None:
            raise Exception("No python-ml-worker enclave spec found for the datalab")
        return (driver_spec, python_spec)
